<?php

namespace App\Controller\Config;

use App\Entity\MaterialType;
use App\Form\Config\MaterialTypeEditForm;
use App\Misc\FlashType;
use App\Repository\MaterialTypeRepository;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Attribute\Route;

/**
 * Controller for the configuration pages of material types
 */
#[Route('/Config/MaterialType')]
class MaterialTypeController extends AbstractController {
    private readonly EntityManagerInterface $entityManager;
    /**
     * @var MaterialTypeRepository The material type repository
     */
    private readonly MaterialTypeRepository $materialTypeRepository;

    /**
     * Initialization
     *
     * @param EntityManagerInterface $entityManager          The entity manager
     * @param MaterialTypeRepository $materialTypeRepository The material type repository
     */
    public function __construct (EntityManagerInterface $entityManager, MaterialTypeRepository $materialTypeRepository) {
        $this->entityManager = $entityManager;
        $this->materialTypeRepository = $materialTypeRepository;
    }

    /**
     * List of material types
     *
     * @return Response The response
     */
    #[Route('/', name: 'config_materialType_list', alias: 'config_materialType')]
    public function list (): Response {
        return $this->render(
            'Config/MaterialType/List.html.twig',
            [
                'materialTypes' => $this->materialTypeRepository->findAll(),
            ]
        );
    }

    /**
     * Edit/Create a material type
     *
     * @param Request           $request      The request
     * @param MaterialType|null $materialType The material type to edit
     *
     * @return Response The response
     */
    #[Route('/Create', name: 'config_materialType_create')]
    #[Route('/Edit-{id}', name: 'config_materialType_edit')]
    public function edit (Request $request, ?MaterialType $materialType = null): Response {
        $materialType ??= new MaterialType();

        $form = $this->createForm(MaterialTypeEditForm::class, $materialType);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $this->entityManager->persist($materialType);
            $this->entityManager->flush();

            $this->addFlash(FlashType::SUCCESS, 'Le type de matériau a bien été enregistré.');
            return $this->redirectToRoute('config_materialType_list');
        }

        return $this->render('Config/MaterialType/Edit.html.twig', [
            'materialType' => $materialType,
            'form'         => $form->createView(),
        ]);
    }

    /**
     * Delete a material type
     *
     * @param MaterialType $materialType The material type to delete
     *
     * @return Response The response
     */
    #[Route('/Delete-{id}', name: 'config_materialType_delete')]
    public function delete (MaterialType $materialType): Response {
        $this->entityManager->remove($materialType);
        $this->entityManager->flush();

        $this->addFlash(FlashType::SUCCESS, 'Le type de matériau a bien été supprimé.');
        return $this->redirectToRoute('config_materialType_list');
    }
}
